@@ -5,7 +5,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #ifndef __XMLSEC_APP_H__
 #define __XMLSEC_APP_H__
@@ -36,9 +36,9 @@ extern "C" {
  * Crypto Init/shutdown
  *
  *********************************************************************/
-XMLSEC_EXPORT int                               xmlSecCryptoInit                (void);
-XMLSEC_EXPORT int                               xmlSecCryptoShutdown            (void);
-XMLSEC_EXPORT int                               xmlSecCryptoKeysMngrInit        (xmlSecKeysMngrPtr mngr);
+XMLSEC_EXPORT int                               xmlSecCryptoInit(void);
+XMLSEC_EXPORT int                               xmlSecCryptoShutdown(void);
+XMLSEC_EXPORT int                               xmlSecCryptoKeysMngrInit(xmlSecKeysMngrPtr mngr);
 
 /*********************************************************************
  *
@@ -51,70 +51,98 @@ XMLSEC_EXPORT int                               xmlSecCryptoKeysMngrInit
  * The AES key klass.
  */
 #define xmlSecKeyDataAesId                      xmlSecKeyDataAesGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataAesGetKlass        (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataAesGetKlass(void);
+/**
+ * xmlSecKeyDataConcatKdfId:
+ *
+ * The ConcatKDF key klass.
+ */
+#define xmlSecKeyDataConcatKdfId                xmlSecKeyDataConcatKdfGetKlass()
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataConcatKdfGetKlass(void);
 /**
  * xmlSecKeyDataDesId:
  *
  * The DES key klass.
  */
 #define xmlSecKeyDataDesId                      xmlSecKeyDataDesGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataDesGetKlass        (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataDesGetKlass(void);
+/**
+ * xmlSecKeyDataDhId:
+ *
+ * The DH key klass.
+ */
+#define xmlSecKeyDataDhId                       xmlSecKeyDataDhGetKlass()
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataDhGetKlass(void);
 /**
  * xmlSecKeyDataDsaId:
  *
  * The DSA key klass.
  */
 #define xmlSecKeyDataDsaId                      xmlSecKeyDataDsaGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataDsaGetKlass        (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataDsaGetKlass(void);
 /**
  * xmlSecKeyDataEcdsaId:
  *
- * The ECDSA key klass.
+ * Deprecated. The EC key klass.
  */
-#define xmlSecKeyDataEcdsaId                    xmlSecKeyDataEcdsaGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataEcdsaGetKlass      (void);
+#define xmlSecKeyDataEcdsaId                    xmlSeckeyDataEcGetKlass()
+XMLSEC_EXPORT XMLSEC_DEPRECATED xmlSecKeyDataId xmlSeckeyDataEcGetKlass(void);
+/**
+ * xmlSecKeyDataEcId:
+ *
+ * The EC key klass.
+ */
+#define xmlSecKeyDataEcId                       xmlSecKeyDataEcetKlass()
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataEcGetKlass(void);
 /**
  * xmlSecKeyDataGost2001Id:
  *
  * The GOST2001 key klass.
  */
 #define xmlSecKeyDataGost2001Id                 xmlSecKeyDataGost2001GetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataGost2001GetKlass   (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataGost2001GetKlass(void);
 /**
  * xmlSecKeyDataGostR3410_2012_256Id:
  *
  * The GOST R 34.10-2012 256 bit key klass.
  */
 #define xmlSecKeyDataGostR3410_2012_256Id       xmlSecKeyDataGostR3410_2012_256GetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataGostR3410_2012_256GetKlass (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataGostR3410_2012_256GetKlass(void);
 /**
  * xmlSecKeyDataGostR3410_2012_512Id:
  *
  * The GOST R 34.10-2012 512 bit key klass.
  */
 #define xmlSecKeyDataGostR3410_2012_512Id       xmlSecKeyDataGostR3410_2012_512GetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataGostR3410_2012_512GetKlass (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataGostR3410_2012_512GetKlass(void);
 /**
  * xmlSecKeyDataHmacId:
  *
- * The DHMAC key klass.
+ * The HMAC key klass.
  */
 #define xmlSecKeyDataHmacId                     xmlSecKeyDataHmacGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataHmacGetKlass       (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataHmacGetKlass(void);
+/**
+ * xmlSecKeyDataPbkdf2Id:
+ *
+ * The PBKDF2 key klass.
+ */
+#define xmlSecKeyDataPbkdf2Id                   xmlSecKeyDataPbkdf2GetKlass()
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataPbkdf2GetKlass(void);
 /**
  * xmlSecKeyDataRsaId:
  *
  * The RSA key klass.
  */
 #define xmlSecKeyDataRsaId                      xmlSecKeyDataRsaGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataRsaGetKlass        (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataRsaGetKlass(void);
 /**
  * xmlSecKeyDataX509Id:
  *
  * The X509 data klass.
  */
 #define xmlSecKeyDataX509Id                     xmlSecKeyDataX509GetKlass()
-XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataX509GetKlass       (void);
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataX509GetKlass(void);
 /**
  * xmlSecKeyDataRawX509CertId:
  *
@@ -122,6 +150,13 @@ XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataX509GetKlass
  */
 #define xmlSecKeyDataRawX509CertId              xmlSecKeyDataRawX509CertGetKlass()
 XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataRawX509CertGetKlass(void);
+/**
+ * xmlSecKeyDataDEREncodedKeyValueId:
+ *
+ * The DEREncodedKeyValue data klass.
+ */
+#define xmlSecKeyDataDEREncodedKeyValueId       xmlSecKeyDataDEREncodedKeyValueGetKlass()
+XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataDEREncodedKeyValueGetKlass(void);
 
 /*********************************************************************
  *
@@ -134,7 +169,7 @@ XMLSEC_EXPORT xmlSecKeyDataId                   xmlSecKeyDataRawX509CertGetKlass
  * The  X509 store klass.
  */
 #define xmlSecX509StoreId                       xmlSecX509StoreGetKlass()
-XMLSEC_EXPORT xmlSecKeyDataStoreId              xmlSecX509StoreGetKlass         (void);
+XMLSEC_EXPORT xmlSecKeyDataStoreId              xmlSecX509StoreGetKlass(void);
 
 /*********************************************************************
  *
@@ -189,92 +224,150 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformAes256GcmGetKlass
  * The AES 128 key wrap transform klass.
  */
 #define xmlSecTransformKWAes128Id               xmlSecTransformKWAes128GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWAes128GetKlass (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWAes128GetKlass(void);
 /**
  * xmlSecTransformKWAes192Id:
  *
  * The AES 192 key wrap transform klass.
  */
 #define xmlSecTransformKWAes192Id               xmlSecTransformKWAes192GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWAes192GetKlass (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWAes192GetKlass(void);
 /**
  * xmlSecTransformKWAes256Id:
  *
  * The AES 256 key wrap transform klass.
  */
 #define xmlSecTransformKWAes256Id               xmlSecTransformKWAes256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWAes256GetKlass (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWAes256GetKlass(void);
+/**
+* xmlSecTransformConcatKdfId:
+*
+* The ConcatKDF key derivation transform klass.
+*/
+#define xmlSecTransformConcatKdfId              xmlSecTransformConcatKdfGetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformConcatKdfGetKlass(void);
 /**
  * xmlSecTransformDes3CbcId:
  *
  * The Triple DES encryption transform klass.
  */
 #define xmlSecTransformDes3CbcId                xmlSecTransformDes3CbcGetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDes3CbcGetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDes3CbcGetKlass(void);
 /**
  * xmlSecTransformKWDes3Id:
  *
  * The DES3 CBC cipher transform klass.
  */
 #define xmlSecTransformKWDes3Id                 xmlSecTransformKWDes3GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWDes3GetKlass   (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformKWDes3GetKlass(void);
+/**
+* xmlSecTransformDhEsId:
+*
+* The DH-ES key agreement transform klass.
+*/
+#define xmlSecTransformDhEsId                   xmlSecTransformDhEsGetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDhEsGetKlass(void);
 /**
  * xmlSecTransformDsaSha1Id:
  *
  * The DSA-SHA1 signature transform klass.
  */
 #define xmlSecTransformDsaSha1Id                xmlSecTransformDsaSha1GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDsaSha1GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDsaSha1GetKlass(void);
 /**
  * xmlSecTransformDsaSha256Id:
  *
- * The DSA-SHA256 signature transform klass.
+ * The DSA-SHA2-256 signature transform klass.
  */
 #define xmlSecTransformDsaSha256Id              xmlSecTransformDsaSha256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDsaSha256GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformDsaSha256GetKlass(void);
+/**
+* xmlSecTransformEcdhId:
+*
+* The ECDH key agreement transform klass.
+*/
+#define xmlSecTransformEcdhId                   xmlSecTransformEcdhGetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdhGetKlass(void);
 /**
  * xmlSecTransformEcdsaSha1Id:
  *
  * The ECDSA-SHA1 signature transform klass.
  */
 #define xmlSecTransformEcdsaSha1Id              xmlSecTransformEcdsaSha1GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha1GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha1GetKlass(void);
+
 /**
  * xmlSecTransformEcdsaSha224Id:
  *
- * The ECDSA-SHA224 signature transform klass.
+ * The ECDSA-SHA2-224 signature transform klass.
  */
 #define xmlSecTransformEcdsaSha224Id            xmlSecTransformEcdsaSha224GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha224GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha224GetKlass(void);
 /**
  * xmlSecTransformEcdsaSha256Id:
  *
- * The ECDSA-SHA256 signature transform klass.
+ * The ECDSA-SHA2-256 signature transform klass.
  */
 #define xmlSecTransformEcdsaSha256Id            xmlSecTransformEcdsaSha256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha256GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha256GetKlass(void);
 /**
  * xmlSecTransformEcdsaSha384Id:
  *
- * The ECDS-SHA384 signature transform klass.
+ * The ECDS-SHA2-384 signature transform klass.
  */
 #define xmlSecTransformEcdsaSha384Id            xmlSecTransformEcdsaSha384GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha384GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha384GetKlass(void);
 /**
  * xmlSecTransformEcdsaSha512Id:
  *
- * The ECDSA-SHA512 signature transform klass.
+ * The ECDSA-SHA2-512 signature transform klass.
  */
 #define xmlSecTransformEcdsaSha512Id            xmlSecTransformEcdsaSha512GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha512GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha512GetKlass(void);
 
+/**
+ * xmlSecTransformEcdsaSha3_224Id:
+ *
+ * The ECDSA-SHA3-224 signature transform klass.
+ */
+#define xmlSecTransformEcdsaSha3_224Id          xmlSecTransformEcdsaSha3_224GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha3_224GetKlass(void);
+/**
+ * xmlSecTransformEcdsaSha3_256Id:
+ *
+ * The ECDSA-SHA3-256 signature transform klass.
+ */
+#define xmlSecTransformEcdsaSha3_256Id          xmlSecTransformEcdsaSha3_256GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha3_256GetKlass(void);
+/**
+ * xmlSecTransformEcdsaSha3_384Id:
+ *
+ * The ECDS-SHA3-384 signature transform klass.
+ */
+#define xmlSecTransformEcdsaSha3_384Id          xmlSecTransformEcdsaSha3_384GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha3_384GetKlass(void);
+/**
+ * xmlSecTransformEcdsaSha3_512Id:
+ *
+ * The ECDSA-SHA3-512 signature transform klass.
+ */
+#define xmlSecTransformEcdsaSha3_512Id          xmlSecTransformEcdsaSha3_512GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaSha3_512GetKlass(void);
+
+/**
+ * xmlSecTransformEcdsaRipemd160Id:
+ *
+ * The ECDSA-RIPEMD160 signature transform klass.
+ */
+#define xmlSecTransformEcdsaRipemd160Id         xmlSecTransformEcdsaRipemd160GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformEcdsaRipemd160GetKlass(void);
 /**
  * xmlSecTransformGost2001GostR3411_94Id:
  *
  * The GOST2001-GOSTR3411_94 signature transform klass.
  */
 #define xmlSecTransformGost2001GostR3411_94Id           xmlSecTransformGost2001GostR3411_94GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGost2001GostR3411_94GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGost2001GostR3411_94GetKlass(void);
 
 /**
  * xmlSecTransformGostR3410_2012GostR3411_2012_256Id:
@@ -282,7 +375,7 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGost2001GostR3411
  * The GOST R 34.10-2012 - GOST R 34.11-2012 256 bit signature transform klass.
  */
 #define xmlSecTransformGostR3410_2012GostR3411_2012_256Id   xmlSecTransformGostR3410_2012GostR3411_2012_256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3410_2012GostR3411_2012_256GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3410_2012GostR3411_2012_256GetKlass(void);
 
 /**
  * xmlSecTransformGostR3410_2012GostR3411_2012_512Id:
@@ -290,7 +383,7 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3410_2012Gos
  * The GOST R 34.10-2012 - GOST R 34.11-2012 512 bit signature transform klass.
  */
 #define xmlSecTransformGostR3410_2012GostR3411_2012_512Id   xmlSecTransformGostR3410_2012GostR3411_2012_512GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3410_2012GostR3411_2012_512GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3410_2012GostR3411_2012_512GetKlass(void);
 
 /**
  * xmlSecTransformHmacMd5Id:
@@ -298,7 +391,7 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3410_2012Gos
  * The HMAC with MD5 signature transform klass.
  */
 #define xmlSecTransformHmacMd5Id                xmlSecTransformHmacMd5GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacMd5GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacMd5GetKlass(void);
 /**
  * xmlSecTransformHmacRipemd160Id:
  *
@@ -312,35 +405,35 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacRipemd160GetK
  * The HMAC with SHA1 signature transform klass.
  */
 #define xmlSecTransformHmacSha1Id               xmlSecTransformHmacSha1GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha1GetKlass (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha1GetKlass(void);
 /**
  * xmlSecTransformHmacSha224Id:
  *
- * The HMAC with SHA224 signature transform klass.
+ * The HMAC with SHA2-224 signature transform klass.
  */
 #define xmlSecTransformHmacSha224Id             xmlSecTransformHmacSha224GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha224GetKlass       (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha224GetKlass(void);
 /**
  * xmlSecTransformHmacSha256Id:
  *
- * The HMAC with SHA256 signature transform klass.
+ * The HMAC with SHA2-256 signature transform klass.
  */
 #define xmlSecTransformHmacSha256Id             xmlSecTransformHmacSha256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha256GetKlass       (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha256GetKlass(void);
 /**
  * xmlSecTransformHmacSha384Id:
  *
- * The HMAC with SHA384 signature transform klass.
+ * The HMAC with SHA2-384 signature transform klass.
  */
 #define xmlSecTransformHmacSha384Id             xmlSecTransformHmacSha384GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha384GetKlass       (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha384GetKlass(void);
 /**
  * xmlSecTransformHmacSha512Id:
  *
- * The HMAC with SHA512 signature transform klass.
+ * The HMAC with SHA2-512 signature transform klass.
  */
 #define xmlSecTransformHmacSha512Id             xmlSecTransformHmacSha512GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha512GetKlass       (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha512GetKlass(void);
 /**
  * xmlSecTransformMd5Id:
  *
@@ -348,6 +441,13 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformHmacSha512GetKlas
  */
 #define xmlSecTransformMd5Id                    xmlSecTransformMd5GetKlass()
 XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformMd5GetKlass(void);
+/**
+* xmlSecTransformPbkdf2Id:
+*
+* The PBKDF2 key derivation transform klass.
+*/
+#define xmlSecTransformPbkdf2Id                 xmlSecTransformPbkdf2GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformPbkdf2GetKlass(void);
 /**
  * xmlSecTransformRipemd160Id:
  *
@@ -361,49 +461,115 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRipemd160GetKlass
  * The RSA-MD5 signature transform klass.
  */
 #define xmlSecTransformRsaMd5Id                 xmlSecTransformRsaMd5GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaMd5GetKlass   (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaMd5GetKlass(void);
 /**
  * xmlSecTransformRsaRipemd160Id:
  *
  * The RSA-RIPEMD160 signature transform klass.
  */
 #define xmlSecTransformRsaRipemd160Id           xmlSecTransformRsaRipemd160GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaRipemd160GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaRipemd160GetKlass(void);
 /**
  * xmlSecTransformRsaSha1Id:
  *
  * The RSA-SHA1 signature transform klass.
  */
 #define xmlSecTransformRsaSha1Id                xmlSecTransformRsaSha1GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha1GetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha1GetKlass(void);
 /**
  * xmlSecTransformRsaSha224Id:
  *
- * The RSA-SHA224 signature transform klass.
+ * The RSA-SHA2-224 signature transform klass.
  */
 #define xmlSecTransformRsaSha224Id              xmlSecTransformRsaSha224GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha224GetKlass        (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha224GetKlass(void);
 /**
  * xmlSecTransformRsaSha256Id:
  *
- * The RSA-SHA256 signature transform klass.
+ * The RSA-SHA2-256 signature transform klass.
  */
 #define xmlSecTransformRsaSha256Id              xmlSecTransformRsaSha256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha256GetKlass        (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha256GetKlass(void);
 /**
  * xmlSecTransformRsaSha384Id:
  *
- * The RSA-SHA384 signature transform klass.
+ * The RSA-SHA2-384 signature transform klass.
  */
 #define xmlSecTransformRsaSha384Id              xmlSecTransformRsaSha384GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha384GetKlass        (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha384GetKlass(void);
 /**
  * xmlSecTransformRsaSha512Id:
  *
- * The RSA-SHA512 signature transform klass.
+ * The RSA-SHA2-512 signature transform klass.
  */
 #define xmlSecTransformRsaSha512Id              xmlSecTransformRsaSha512GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha512GetKlass        (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha512GetKlass(void);
+
+/**
+ * xmlSecTransformRsaPssSha1Id:
+ *
+ * The RSA-PSS-SHA1 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha1Id             xmlSecTransformRsaPssSha1GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha1GetKlass(void);
+
+/**
+ * xmlSecTransformRsaPssSha224Id:
+ *
+ * The RSA-PSS-SHA2-224 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha224Id           xmlSecTransformRsaPssSha224GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha224GetKlass(void);
+/**
+ * xmlSecTransformRsaPssSha256Id:
+ *
+ * The RSA-PSS-SHA2-256 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha256Id           xmlSecTransformRsaPssSha256GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha256GetKlass(void);
+/**
+ * xmlSecTransformRsaPssSha384Id:
+ *
+ * The RSA-PSS-SHA2-384 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha384Id           xmlSecTransformRsaPssSha384GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha384GetKlass(void);
+/**
+ * xmlSecTransformRsaPssSha512Id:
+ *
+ * The RSA-PSS-SHA2-512 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha512Id           xmlSecTransformRsaPssSha512GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha512GetKlass(void);
+
+/**
+ * xmlSecTransformRsaPssSha3_224Id:
+ *
+ * The RSA-PSS-SHA3-224 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha3_224Id         xmlSecTransformRsaPssSha3_224GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha3_224GetKlass(void);
+/**
+ * xmlSecTransformRsaPssSha3_256Id:
+ *
+ * The RSA-PSS-SHA3-256 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha3_256Id         xmlSecTransformRsaPssSha3_256GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha3_256GetKlass(void);
+/**
+ * xmlSecTransformRsaPssSha3_384Id:
+ *
+ * The RSA-PSS-SHA3-384 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha3_384Id         xmlSecTransformRsaPssSha3_384GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha3_384GetKlass(void);
+/**
+ * xmlSecTransformRsaPssSha3_512Id:
+ *
+ * The RSA-PSS-SHA3-512 signature transform klass.
+ */
+#define xmlSecTransformRsaPssSha3_512Id         xmlSecTransformRsaPssSha3_512GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPssSha3_512GetKlass(void);
 
 /**
  * xmlSecTransformRsaPkcs1Id:
@@ -411,35 +577,42 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaSha512GetKlass
  * The RSA PKCS1 key transport transform klass.
  */
 #define xmlSecTransformRsaPkcs1Id               xmlSecTransformRsaPkcs1GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPkcs1GetKlass (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaPkcs1GetKlass(void);
 /**
  * xmlSecTransformRsaOaepId:
  *
- * The RSA PKCS1 key transport transform klass.
+ * The RSA PKCS1 key transport transform klass (XMLEnc 1.0).
  */
 #define xmlSecTransformRsaOaepId                xmlSecTransformRsaOaepGetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaOaepGetKlass  (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaOaepGetKlass(void);
+/**
+ * xmlSecTransformRsaOaepEnc11Id:
+ *
+ * The RSA PKCS1 key transport transform klass (XMLEnc 1.1).
+ */
+#define xmlSecTransformRsaOaepEnc11Id           xmlSecTransformRsaOaepEnc11GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformRsaOaepEnc11GetKlass(void);
 /**
  * xmlSecTransformGostR3411_94Id:
  *
  * The GOSTR3411_94 digest transform klass.
  */
-#define xmlSecTransformGostR3411_94Id                   xmlSecTransformGostR3411_94GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_94GetKlass     (void);
+#define xmlSecTransformGostR3411_94Id           xmlSecTransformGostR3411_94GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_94GetKlass(void);
 /**
  * xmlSecTransformGostR3411_2012_256Id:
  *
  * The GOST R 34.11-2012 256 bit digest transform klass.
  */
 #define xmlSecTransformGostR3411_2012_256Id     xmlSecTransformGostR3411_2012_256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_2012_256GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_2012_256GetKlass(void);
 /**
  * xmlSecTransformGostR3411_2012_512Id:
  *
  * The GOST R 34.11-2012 512 bit digest transform klass.
  */
 #define xmlSecTransformGostR3411_2012_512Id     xmlSecTransformGostR3411_2012_512GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_2012_512GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_2012_512GetKlass(void);
 
 /**
  * xmlSecTransformSha1Id:
@@ -447,35 +620,66 @@ XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformGostR3411_2012_51
  * The SHA1 digest transform klass.
  */
 #define xmlSecTransformSha1Id                   xmlSecTransformSha1GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha1GetKlass     (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha1GetKlass(void);
+
 /**
  * xmlSecTransformSha224Id:
  *
- * The SHA224 digest transform klass.
+ * The SHA2-224 digest transform klass.
  */
 #define xmlSecTransformSha224Id                 xmlSecTransformSha224GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha224GetKlass   (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha224GetKlass(void);
 /**
  * xmlSecTransformSha256Id:
  *
- * The SHA256 digest transform klass.
+ * The SHA2-256 digest transform klass.
  */
 #define xmlSecTransformSha256Id                 xmlSecTransformSha256GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha256GetKlass   (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha256GetKlass(void);
 /**
  * xmlSecTransformSha384Id:
  *
- * The SHA384 digest transform klass.
+ * The SHA2-384 digest transform klass.
  */
 #define xmlSecTransformSha384Id                 xmlSecTransformSha384GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha384GetKlass   (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha384GetKlass(void);
 /**
  * xmlSecTransformSha512Id:
  *
- * The SHA512 digest transform klass.
+ * The SHA2-512 digest transform klass.
  */
 #define xmlSecTransformSha512Id                 xmlSecTransformSha512GetKlass()
-XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha512GetKlass   (void);
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha512GetKlass(void);
+
+
+/**
+ * xmlSecTransformSha3_224Id:
+ *
+ * The SHA3-224 digest transform klass.
+ */
+#define xmlSecTransformSha3_224Id               xmlSecTransformSha3_224GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha3_224GetKlass(void);
+/**
+ * xmlSecTransformSha3_256Id:
+ *
+ * The SHA3-256 digest transform klass.
+ */
+#define xmlSecTransformSha3_256Id               xmlSecTransformSha3_256GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha3_256GetKlass(void);
+/**
+ * xmlSecTransformSha3_384Id:
+ *
+ * The SHA3-384 digest transform klass.
+ */
+#define xmlSecTransformSha3_384Id               xmlSecTransformSha3_384GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha3_384GetKlass(void);
+/**
+ * xmlSecTransformSha3_512Id:
+ *
+ * The SHA3-512 digest transform klass.
+ */
+#define xmlSecTransformSha3_512Id               xmlSecTransformSha3_512GetKlass()
+XMLSEC_EXPORT xmlSecTransformId                 xmlSecTransformSha3_512GetKlass(void);
 
 /*********************************************************************
  *
@@ -487,6 +691,9 @@ XMLSEC_EXPORT int                               xmlSecCryptoAppShutdown
 XMLSEC_EXPORT int                               xmlSecCryptoAppDefaultKeysMngrInit      (xmlSecKeysMngrPtr mngr);
 XMLSEC_EXPORT int                               xmlSecCryptoAppDefaultKeysMngrAdoptKey  (xmlSecKeysMngrPtr mngr,
                                                                                          xmlSecKeyPtr key);
+XMLSEC_EXPORT int                               xmlSecCryptoAppDefaultKeysMngrVerifyKey (xmlSecKeysMngrPtr mngr,
+                                                                                         xmlSecKeyPtr key,
+                                                                                         xmlSecKeyInfoCtxPtr keyInfoCtx);
 XMLSEC_EXPORT int                               xmlSecCryptoAppDefaultKeysMngrLoad      (xmlSecKeysMngrPtr mngr,
                                                                                          const char* uri);
 XMLSEC_EXPORT int                               xmlSecCryptoAppDefaultKeysMngrSave      (xmlSecKeysMngrPtr mngr,
@@ -501,7 +708,20 @@ XMLSEC_EXPORT int                               xmlSecCryptoAppKeysMngrCertLoadM
                                                                                  xmlSecSize dataSize,
                                                                                  xmlSecKeyDataFormat format,
                                                                                  xmlSecKeyDataType type);
-XMLSEC_EXPORT xmlSecKeyPtr                      xmlSecCryptoAppKeyLoad          (const char *filename,
+XMLSEC_EXPORT int                               xmlSecCryptoAppKeysMngrCrlLoad  (xmlSecKeysMngrPtr mngr,
+                                                                                 const char *filename,
+                                                                                 xmlSecKeyDataFormat format);
+XMLSEC_EXPORT int                               xmlSecCryptoAppKeysMngrCrlLoadMemory(xmlSecKeysMngrPtr mngr,
+                                                                                 const xmlSecByte* data,
+                                                                                 xmlSecSize dataSize,
+                                                                                 xmlSecKeyDataFormat format);
+XMLSEC_DEPRECATED XMLSEC_EXPORT xmlSecKeyPtr    xmlSecCryptoAppKeyLoad          (const char *filename,
+                                                                                 xmlSecKeyDataFormat format,
+                                                                                 const char *pwd,
+                                                                                 void* pwdCallback,
+                                                                                 void* pwdCallbackCtx);
+XMLSEC_EXPORT xmlSecKeyPtr                      xmlSecCryptoAppKeyLoadEx        (const char *filename,
+                                                                                 xmlSecKeyDataType type,
                                                                                  xmlSecKeyDataFormat format,
                                                                                  const char *pwd,
                                                                                  void* pwdCallback,
@@ -537,4 +757,3 @@ XMLSEC_EXPORT void*                             xmlSecCryptoAppGetDefaultPwdCall
 #endif /* XMLSEC_NO_CRYPTO_DYNAMIC_LOADING */
 
 #endif /* __XMLSEC_APP_H__ */
-
